@@ -13,7 +13,7 @@ use Perinci::Object;
 use Perinci::ToUtil;
 use Scalar::Util qw(reftype blessed);
 
-our $VERSION = '1.06'; # VERSION
+our $VERSION = '1.07'; # VERSION
 
 our $REQ_VERSION = 0; # version requested by user
 
@@ -987,8 +987,13 @@ sub help_section_options {
                 my $alspec = $a->{cmdline_aliases}{$al0};
                 next if $alspec->{code};
                 my $al = $al0; $al =~ s/_/-/g;
-                $al = length($al) > 1 ? "--$al" : "-$al";
-                $ane .= ", $al$suf";
+                if (length($al) == 1) {
+                    $al = "-$al";
+                    $ane .= ", $al";
+                } else {
+                    $al = "--$al";
+                    $ane .= ", $al$suf";
+                }
             }
 
             my $def = defined($s->[1]{default}) && $s->[0] ne 'bool' ?
@@ -1438,7 +1443,15 @@ sub run_call {
 
     my $tx_id;
 
-    my $using_tx = !$self->{_dry_run} && $self->undo && ($sc->{undo} // 1);
+    my $dry_run = $self->{_dry_run};
+    my $using_tx = !$dry_run && $self->undo && ($sc->{undo} // 1);
+
+    # currently we don't attempt to insert tx_id or dry_run when using argv,
+    # we'll just give up
+    if ($self->{_send_argv} && ($dry_run || $using_tx)) {
+        my $res = $self->{_getargs_result};
+        $self->_err("Failed parsing arguments (2): $res->[0] - $res->[1]");
+    }
 
     if ($using_tx) {
         require UUID::Random;
@@ -1460,9 +1473,16 @@ sub run_call {
     }
 
     # call function
-    $self->{_res} = $self->_pa->request(
-        call => $self->{_subcommand}{url},
-        {args=>\%fargs, tx_id=>$tx_id, dry_run=>$self->{_dry_run}});
+    if ($self->{_send_argv}) {
+        $self->{_res} = $self->_pa->request(
+            call => $self->{_subcommand}{url},
+            {argv=>$self->{_orig_argv}}, # XXX tx_id, dry_run (see above)
+        );
+    } else {
+        $self->{_res} = $self->_pa->request(
+            call => $self->{_subcommand}{url},
+            {args=>\%fargs, tx_id=>$tx_id, dry_run=>$dry_run});
+    }
     $log->tracef("call res=%s", $self->{_res});
 
     # commit transaction (if using tx)
@@ -1639,6 +1659,16 @@ sub parse_subcommand_opts {
         $self->_load_log_any_app if $do_log;
     }
 
+    # we'll try giving argv to server side, but this currently means we skip
+    # processing cmdline_src.
+    if ($res->[0] == 502) {
+        $log->debugf("Failed parsing arguments (status 502), will try to send ".
+                         "argv to server");
+        $self->{_getargs_result} = $res;
+        $self->{_send_argv} = 1;
+        return;
+    }
+
     $self->_err("Failed parsing arguments: $res->[0] - $res->[1]")
         unless $res->[0] == 200;
     for (keys %{ $res->[2] }) {
@@ -1888,7 +1918,7 @@ Perinci::CmdLine - Rinci/Riap-based command-line application framework
 
 =head1 VERSION
 
-version 1.06
+version 1.07
 
 =head1 SYNOPSIS
 
